using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Xml;

using EQIA.EQIDL;

namespace EQIA
{
    public class EQInterfaceFile
    {
        #region Fields

        private EQInterface myInterface;

        private FileInfo myFile;
        private XmlDocument myXmlDocument;

        private List<string> myElements;

        #endregion

        #region Properties

        /// <summary>
        /// The interface this file belongs to.
        /// </summary>
        [Browsable(true), Description("The interface this file belongs to.")]
        public EQInterface Interface { get { return myInterface; } set { myInterface = value; } }

        /// <summary>
        /// The name of this file.
        /// </summary>
        [Browsable(true), Description("The name of the file on disk this equi file resides in.")]
        public string Filename { get { return myFile != null ? myFile.Name : string.Empty; } }

        /// <summary>
        /// A list of elements that are found in this file.
        /// </summary>
        [Browsable(true), Description("List of elements found in this equi file.")]
        public List<string> Elements { get { return myElements; } }

        #endregion

        #region Constructor

        public EQInterfaceFile(EQInterface eqInterface, string filename)
        {
            myInterface = eqInterface;
            myFile = new FileInfo(filename);
            myXmlDocument = new XmlDocument();
            myElements = new List<string>();
        }

        #endregion

        #region Listing Methods

        public void AddElement(Class newClass)
        {
            myElements.Add(newClass.Name);
            myInterface.AddElement(newClass);
        }

        public void RemoveElement(string name)
        {
            myElements.Remove(name);
            myInterface.RemoveElement(name);
        }

        public Class GetElement(string name)
        {
            return myInterface != null ? myInterface.GetElement(name) : null;
        }

        public void SwapElements(string first, string second)
        {
            if (myElements.Contains(first) && myElements.Contains(second))
            {
                int firstIndex = myElements.IndexOf(first);
                int secondIndex = myElements.IndexOf(second);

                myElements[firstIndex] = second;
                myElements[secondIndex] = first;
            }
        }

        public void MoveElement(int source, int dest)
        {
            if (source < myElements.Count && dest < myElements.Count)
            {
                string temp = myElements[source];
                myElements.Remove(temp);
                myElements.Insert(dest - 1, temp);
            }
        }

        #endregion

        #region Xml Methods

        public void Deserialize()
        {
            myXmlDocument = new XmlDocument();
            if (myFile.Exists) myXmlDocument.Load(myFile.FullName);
            else return;

            XmlNode xnRoot = myXmlDocument.DocumentElement;

            // Loop through the child nodes of the root node
            for (int i = 0; i < xnRoot.ChildNodes.Count; i++)
            {
                // Check if the current node is an element
                if (xnRoot.ChildNodes[i].NodeType == XmlNodeType.Element)
                {
                    // A class that will later be transformed to the proper type
                    Class newClass = new Class(myInterface);

                    #region Element Discovery - Add new element types here
                    // Now find what it really is, convert it, and finalize it
                    switch (xnRoot.ChildNodes[i].LocalName)
                    {
                        case "Button":
                            newClass = new Button(myInterface);
                            goto case "FinalizeControl";

                        case "ButtonDrawTemplate":
                            newClass = new ButtonDrawTemplate(myInterface);
                            goto case "FinalizeControl";

                        case "Combobox":
                            newClass = new Combobox(myInterface);
                            goto case "FinalizeControl";

                        case "Editbox":
                            newClass = new Editbox(myInterface);
                            goto case "FinalizeControl";

                        case "Finderbox":
                            newClass = new FinderBox(myInterface);
                            goto case "FinalizeControl";

                        case "FrameTemplate":
                            newClass = new FrameTemplate(myInterface);
                            goto case "FinalizeControl";

                        case "Gauge":
                            newClass = new Gauge(myInterface);
                            goto case "FinalizeControl";

                        case "GaugeDrawTemplate":
                            newClass = new GaugeDrawTemplate(myInterface);
                            goto case "FinalizeControl";

                        case "HorizontalLayoutBox":
                            newClass = new HorizontalLayoutBox(myInterface);
                            goto case "FinalizeControl";

                        case "InvSlot":
                            newClass = new InvSlot(myInterface);
                            goto case "FinalizeControl";

                        case "Label":
                            newClass = new Label(myInterface);
                            goto case "FinalizeControl";

                        case "LayoutBox":
                            newClass = new LayoutBox(myInterface);
                            goto case "FinalizeControl";

                        case "Listbox":
                            newClass = new Listbox(myInterface);
                            goto case "FinalizeControl";

                        case "Page":
                            newClass = new Page(myInterface);
                            goto case "FinalizeControl";

                        case "Screen":
                            newClass = new Screen(myInterface);
                            goto case "FinalizeControl";

                        case "Slider":
                            newClass = new Slider(myInterface);
                            goto case "FinalizeControl";

                        case "SliderDrawTemplate":
                            newClass = new SliderDrawTemplate(myInterface);
                            goto case "FinalizeControl";

                        case "SpellGem":
                            newClass = new SpellGem(myInterface);
                            goto case "FinalizeControl";

                        case "SpellGemDrawTemplate":
                            newClass = new SpellGemDrawTemplate(myInterface);
                            goto case "FinalizeControl";

                        case "StaticAnimation":
                            newClass = new StaticAnimation(myInterface);
                            goto case "FinalizeControl";

                        case "StaticFrame":
                            newClass = new StaticFrame(myInterface);
                            goto case "FinalizeControl";

                        case "StaticHeader":
                            newClass = new StaticHeader(myInterface);
                            goto case "FinalizeControl";

                        case "StaticText":
                            newClass = new StaticText(myInterface);
                            goto case "FinalizeControl";

                        case "StaticTintedBlendAnimation":
                            newClass = new StaticTintedBlendAnimation(myInterface);
                            goto case "FinalizeControl";

                        case "STMLBox":
                            newClass = new STMLbox(myInterface);
                            goto case "FinalizeControl";

                        case "TabBox":
                            newClass = new TabBox(myInterface);
                            goto case "FinalizeControl";

                        case "TextureInfo":
                            newClass = new TextureInfo(myInterface);
                            goto case "FinalizeControl";

                        case "Ui2DAnimation":
                            newClass = new Ui2DAnimation(myInterface);
                            goto case "FinalizeControl";

                        case "VerticalLayoutBox":
                            newClass = new VerticalLayoutBox(myInterface);
                            goto case "FinalizeControl";

                        case "WindowDrawTemplate":
                            newClass = new WindowDrawTemplate(myInterface);
                            goto case "FinalizeControl";

                        case "FinalizeControl":
                            {
                                newClass.LoadXml(xnRoot.ChildNodes[i].OuterXml);    // Load the xml fragment into the control
                                newClass.Deserialize();                             // Parse the xml document
                                AddElement(newClass);                               // Add the new control to the list
                            }
                            break;
                    }
                    #endregion
                }
            }
        }

        public void Serialize()
        {
            // Create the string that will hold the xml, add the header data here too
            string xml = "<?xml version=\"1.0\" encoding=\"us-ascii\" ?>" +
                         "<XML ID=\"EQInterfaceDefinitionLanguage\">" +
                         "<Schema xmlns=\"EverQuestData\" xmlns:dt=\"EverQuestDataTypes\" />";

            // Loop through the elements in the list
            for (int i = 0; i < myElements.Count; i++)
            {
                Class element = myInterface.GetElement(myElements[i]);
                element.Serialize();

                // Add the xml from the current element to the xml stream
                xml += element.XmlToString();
            }

            xml += "</XML>";    // Add the footer to the xml document
            
            myXmlDocument = new XmlDocument();
            myXmlDocument.LoadXml(xml);
        }

        #endregion
    }
}
